@@ -7,7 +7,7 @@ use Log::Any::Adapter::Util 'numeric_level';
 use Log::Any 1.03;
 use Log::Any::Adapter::Daemontools::Config;
 
-our $VERSION= '0.0900000_003';
+our $VERSION= '0.0900000_004';
 
 # ABSTRACT: Logging adapter suitable for use in a Daemontools-style logging chain
 
@@ -29,12 +29,21 @@ sub category { shift->{category} }
 sub config   { shift->{config} }
 
 
+# Special carp/croak that ignore Log::Any infrastructure
+sub carp { Log::Any::Adapter::Daemontools::Config::carp(@_) }
+sub croak { Log::Any::Adapter::Daemontools::Config::croak(@_) }
+
 # Log::Any::Adapter constructor, also named 'init'
 sub init {
 	my $self= shift;
 	
 	$self->{config} ||= $self->global_config;
 	
+	# Warn about unsupported/deprecated features from 0.002
+	carp "filter is deprecated.  Use config->log_level" if exists $self->{filter};
+	carp "dumper is unsupported. See Log::Any::Proxy" if exists $self->{dumper};
+	
+	
 	# This constructor gets called for each Adapter instance, so we need
 	# to track whether we applied the -init to the config yet.
 	if ($self->{'-init'} && !$self->{config}{_adapter_init_applied}) {
@@ -105,6 +114,10 @@ sub _cache_config {
 	my $self= shift;
 	$self->{_writer}= $self->config->compiled_writer;
 	my $lev= $self->config->log_level_num;
+	# Backward compatibility with version 0.002
+	if (exists $self->{filter}) {
+		$lev= Log::Any::Adapter::Util::NOTICE - _coerce_filter_level($self->{filter});
+	}
 	bless $self, $self->_squelch_base_class.'::Squelch'.($lev+1);
 	$self->config->_register_cached_adapter($self);
 }
@@ -114,6 +127,48 @@ sub _uncache_config {
 	bless $_[0], $_[0]->_squelch_base_class . '::Lazy';
 }
 
+#-------------------------------------------------------------------
+# Backward compatibility with version 0.002.  Do not use in new code.
+
+sub write_msg {
+	my ($self, $level, $message)= @_;
+	# Don't bother optimizing and caching
+	$self->config->compiled_writer->($self, $level, $message);
+}
+
+sub _default_dumper {
+	require Data::Dumper;
+	my $val= shift;
+	local $@;
+	my $dump= eval { Data::Dumper->new([$val])->Indent(0)->Terse(1)->Useqq(1)->Quotekeys(0)->Maxdepth(4)->Sortkeys(1)->Dump };
+	if (!defined $dump) {
+		my $x= "$@";
+		$x =~ s/\n//;
+		substr($x, 50)= '...' if length $x >= 50;
+		$dump= "<exception $x>";
+	};
+	return $dump;
+}
+
+sub _coerce_filter_level {
+	my $val= shift;
+	my %level_map= (
+		trace    => -2,
+		debug    => -1,
+		info     =>  0,
+		notice   =>  1,
+		warning  =>  2,
+		error    =>  3,
+		critical =>  4,
+		fatal    =>  4,
+	);
+	return (!defined $val || $val eq 'none')? $level_map{trace}-1
+		: Scalar::Util::looks_like_number($val)? $val
+		: exists $level_map{$val}? $level_map{$val}
+		: ($val =~ /^debug-(\d+)$/)? $level_map{debug} - $1
+		: croak "unknown log level '$val'";
+}
+
 1;
 
 __END__
@@ -128,45 +183,44 @@ Log::Any::Adapter::Daemontools - Logging adapter suitable for use in a Daemontoo
 
 =head1 VERSION
 
-version 0.090000_003
+version 0.090000_004
 
 =head1 SYNOPSIS
 
-  # Log to STDOUT with log-level-name prefixes for all but 'info'
-  # Default log level is 'info'
+Default: log to C<STDOUT>, log level C<info>, prefix each line with level name unless it is C<info>:
+
   use Log::Any::Adapter 'Daemontools';
-  
-  # As above, but log level 'notice'
-  use Log::Any::Adapter 'Daemontools', init => { level => 'notice' };
-  
-  # As above, but process @ARGV -v/-q and $ENV{DEBUG} to adjust the level
-  use Log::Any::Adapter 'Daemontools', init => { argv => 1, env => 1 };
-  
-  # Direct edits to shared config
+
+Default to log level C<notice>, and write to C<STDERR>:
+
+  use Log::Any::Adapter 'Daemontools', -init => { level => 'notice', out => \*STDERR };
+
+Process C<@ARGV> C<-v>/C<-q> and C<$ENV{DEBUG}> to adjust the log level
+
+  use Log::Any::Adapter 'Daemontools', -init => { argv => 1, env => 1 };
+
+Custom output formatting:
+
+  use Log::Any::Adapter 'Daemontools', -init => { format => '"$level: $_ (at $file on line $line)"' };
+
+Change log level on the fly:
+
   my $cfg= Log::Any::Adapter::Daemontools->global_config;
-  $cfg->log_level('debug');
-  $cfg->init( argv => 1, env => 1 );
-  
-  # Change log level on the fly
   $SIG{USR1}= sub { $cfg->log_level_adjust(1); };
   $SIG{USR2}= sub { $cfg->log_level_adjust(-1); };
   
-  # Signal handlers can also be installed by the config:
-  $cfg->init( handle_signals => ['USR1','USR2'] );
-  
-  # Create a second config independent of the global config
-  my $cfg2= Log::Any::Adapter::Daemontools->new_config;
-  
-  # Multiple adapter configurations, tracking different config instances
-  Log::Any::Adapter->set({ category => qr/^Noisy::Package.*/ }, 'Daemontools', config => $cfg2 );
-  Log::Any::Adapter->set('Daemontools'); # config defaults to global_config
-  $cfg2->log_level('warn'); # lower log level for messages from Noisy::Package::*
-  
-  # Like above, but limit the verbosity instead of creating a second config
-  Log::Any::Adapter->set({ category => qr/^Noisy::Package.*/ }, 'Daemontools', log_level_max => 'warn' );
-  Log::Any::Adapter->set('Daemontools'); # config defaults to global_config
+  # Those signal handlers can also be installed by the config:
+  use Log::Any::Adapter 'Daemontools', -init => { signals => ['USR1','USR2'] };
 
-See L<Log::Any::Adapter::Daemontools::Config> for most of the details.
+Create alternate configurations separate from the global config: 
+
+  my $cfg2= Log::Any::Adapter::Daemontools->new_config(log_level => 'notice');
+  Log::Any::Adapter->set(
+    { category => qr/^Noisy::Package::Hierarchy.*/ },
+    config => $cfg2
+  );
+
+See L<Log::Any::Adapter::Daemontools::Config> for most of the usage details.
 
 =head1 DESCRIPTION
 
@@ -174,7 +228,7 @@ The measure of good software is low learning curve, low complexity, few
 dependencies, high efficiency, and high flexibility.  (choose two.  haha)
 
 In the daemontools way of thinking, a daemon writes all its logging output
-to STDOUT (or STDERR), which is a pipe to a logger process.
+to C<STDOUT> (or C<STDERR>), which is a pipe to a logger process.
 Doing this instead of other logging alternatives keeps your program simple
 and allows you to capture errors generated by deeper libraries (like libc)
 which aren't aware of your logging API.  If you want complicated logging you
@@ -182,7 +236,7 @@ can keep those details in the logging process and not bloat each daemon you
 write.
 
 This module aims to be the easiest, simplest, most efficent way to get
-Log::Any messages to a file handle while still being flexible enough for the
+L<Log::Any> messages to a file handle while still being flexible enough for the
 needs of the typical unix daemon or utility script.
 
 Problems solved by this module are:
@@ -194,7 +248,7 @@ Problems solved by this module are:
 The downside of logging to a pipe is you don't get the log-level that you
 could have had with syslog or Log4perl.  An simple way to preserve this
 information is to prefix each line with "error:" or etc, which can be
-re-parsed later (or by the logger process). See L<prefix>.
+re-parsed later (or by the logger process). See L<format|Log::Any::Adapter::Daemontools::Config/format>.
 
 =item Efficiently squelch log levels
 
@@ -211,8 +265,8 @@ from the Log::Any::Proxy, which is unfortunate)
 
 =item Dynamically adjust log levels
 
-Log::Any::Adapter allows you to replace the current adapters with new ones
-with a different configuration, which you can use to adjust log_level,
+L<Log::Any::Adapter> allows you to replace the current adapters with new ones
+with a different configuration, which you can use to adjust C<log_level>,
 but it isn't terribly efficient, and if you are also using the regex feature
 (where different categories get different loggers) it's even worse.
 
@@ -220,7 +274,7 @@ This module uses shared configurations on the back-end so you can alter the
 configuration in many ways without having to re-attach the adapters.
 (there is a small re-caching penalty, but it's done lazily)
 
-=item --verbose / --quiet / $ENV{DEBUG}
+=item C<--verbose> / C<--quiet> / C<$ENV{DEBUG}>
 
 My scripts usually end up with a chunk of boilerplate in the option processing
 to raise or lower the log level.  This module provides an option to get you
@@ -228,12 +282,12 @@ common UNIX behavior in as little as 7 characters :-)
 It's flexible enough to give you many other common varieties, or you can ignore
 it because it isn't enabled by default.
 
-=item Display caller() or category, or custom formatting
+=item Display C<caller> or C<category>, or custom formatting
 
 And of course, you often want to see additional details about the message or
-perform some of your own tweaks.  This module provides quick options to enable
-caller() info and/or category name where the message originated, and allows
-full customization with coderefs.
+perform some of your own tweaks.  This module provides a C<format> option to
+easily add C<caller> info and/or C<category> where the message originated,
+and allows full customization with coderefs.
 
 =back
 
@@ -245,6 +299,29 @@ I don't know if anyone was using them anyway, but pay close attention
 if you are upgrading.  This new version adheres more closely to the
 specification for a logging adapter.
 
+=head1 PACKAGE METHODS
+
+=head2 global_config
+
+  my $cfg= Log::Any::Adapter::Daemontools->global_config;
+
+Returns the default config instance used by any adapter where you didn't
+specify one.
+
+=head2 new_config
+
+  my $cfg= Log::Any::Adapter::Daemontools->new_config( %attributes )
+
+Returns a new instance of a config object appropriate for use with this
+adapter, but currently always an instance of L<Log::Any::Adapter::Daemontools::Config>.
+See that package for available attributes.
+
+  Log::Any::Adapter->set( 'Daemontools', config => $cfg );
+
+This method is preferred over calling ->new on the L<Log::Any::Adapter::Daemontools::Config>
+package directly, in case some day I decide to play subclassing tricks with the
+Config objects.
+
 =head1 ATTRIBUTES
 
 =head2 category
@@ -275,6 +352,9 @@ The implied init() call will happen exactly once per config object.
 See L<Log::Any::Adapter::Daemontools::Config/init> for the complete list
 of initialization options.
 
+DO NOT pass un-sanitized user input to -init, because the 'format' attribute
+is processed as perl code.
+
 =head1 METHODS
 
 Adapter instances support all the standard logging methods of Log::Any::Adapter
