@@ -5,6 +5,8 @@ use warnings;
 # ABSTRACT: Dynamic configuration settings used by Daemontools logging adapters
 
 
+use Log::Any::Adapter::Util 'numeric_level', ':levels';
+
 # At top of file where lexical scope is the cleanest
 sub _build_writer_eval_in_clean_scope {
 	# Args: $self, $code, \$err
@@ -16,7 +18,6 @@ sub _build_writer_eval_in_clean_scope {
 	return $coderef;
 }
 
-use Log::Any::Adapter::Util 'numeric_level', ':levels';
 use Scalar::Util 'weaken', 'refaddr';
 
 # Lazy-load carp, and also remove any Log::Any infrastructure from the trace
@@ -43,6 +44,7 @@ BEGIN {
 		info => INFO,
 		debug => DEBUG,
 		trace => TRACE,
+		all => TRACE
 	);
 	%log_level_name= reverse %log_level_num;
 	
@@ -55,9 +57,9 @@ BEGIN {
 
 sub _parse_log_level {
 	my ($spec, $base, $min, $max)= @_;
-	my $lev= $spec =~ /^-?\d+$/?         $spec
-		: $spec =~ /^([-+])= (-?\d+)$/?  $base + "${1}1" * $2
-		: $log_level_num{$spec};
+	my $lev= $spec =~ /^-?\d+$/? $spec  # plain level
+		: $spec =~ /^([-+])= (-?\d+)$/? $base + "${1}1" * $2 # += notation
+		: $log_level_num{$spec}; # else a level name
 	defined $lev or croak "Invalid log level '$spec'";
 	$min= EMERGENCY-1 unless defined $min;
 	$lev= $min unless $lev >= $min;
@@ -71,6 +73,8 @@ sub log_level {
 	if (@_) {
 		croak "extra arguments" if @_ > 1;
 		my $l= _parse_log_level($_[0], $self->{log_level_num}, $self->{log_level_min_num}, $self->{log_level_max_num});
+
+		# If log level changes, reset the cache
 		if ($l != $self->{log_level_num}) {
 			$self->{log_level_num}= $l;
 			$self->_reset_cached_adapters;
@@ -83,6 +87,8 @@ sub log_level_min {
 	my $self= shift;
 	if (@_) {
 		croak "extra arguments" if @_ > 1;
+		
+		# If log level changes as a result, reset the cache
 		$self->{log_level_min_num}= _parse_log_level($_[0], $self->{log_level_min_num}, EMERGENCY-1, $self->{log_level_max_num});
 		if ($self->{log_level_min_num} > $self->{log_level_num}) {
 			$self->{log_level_num}= $self->{log_level_min_num};
@@ -97,6 +103,8 @@ sub log_level_max {
 	if (@_) {
 		croak "extra arguments" if @_ > 1;
 		$self->{log_level_max_num}= _parse_log_level($_[0], $self->{log_level_max_num}, $self->{log_level_min_num}, TRACE);
+		
+		# If log level changes as a result, reset the cache
 		if ($self->{log_level_max_num} < $self->{log_level_num}) {
 			$self->{log_level_num}= $self->{log_level_max_num};
 			$self->_reset_cached_adapters;
@@ -193,7 +201,7 @@ BEGIN {
 
 my %_init_args= map { $_ => 1 } qw(
 	level log_level min level_min log_level_min max level_max log_level_max
-	env argv signals format output writer
+	env argv signals handle_signals install_signal_handlers format out output writer
 );
 sub init {
 	my $self= shift;
@@ -212,38 +220,36 @@ sub init {
 		for qw: max level_max log_level_max :;
 
 	# Optional ENV processing
-	if ($cfg->{env}) {
+	defined $cfg->{$_} and do {
+		my $v= $cfg->{$_};
 		$self->process_env( %{
-			ref $cfg->{env} eq 'HASH'? $cfg->{env}
-			: $env_profile{$cfg->{env}}
-				|| croak "Unknown \"env\" value $cfg->{env}"
+			ref $v eq 'HASH'? $v : $env_profile{$v} || croak "Unknown \"$_\" value $v"
 		} );
-	}
+	} for qw: env process_env :;
 	
 	# Optional ARGV parsing
-	if ($cfg->{argv}) {
+	defined $cfg->{$_} and do {
+		my $v= $cfg->{$_};
 		$self->process_argv( %{
-			ref $cfg->{argv} eq 'HASH'? $cfg->{argv}
-			: $argv_profile{$cfg->{argv}}
-				|| croak "Unknown \"argv\" value $cfg->{argv}"
+			ref $v eq 'HASH'? $v : $argv_profile{$v} || croak "Unknown \"$_\" value $v"
 		} );
-	}
+	} for qw: argv process_argv :;
 	
 	# Optional installation of signal handlers
-	if ($cfg->{signals}) {
-		my $r= ref($cfg->{signals}) || '';
+	defined $cfg->{$_} and do {
+		my $rt= ref($cfg->{$_}) || '';
 		$self->install_signal_handlers( %{
-			$r eq 'HASH'? $cfg->{signals}
-			: $r eq 'ARRAY'? { verbose => $cfg->{signals}[0], quiet => $cfg->{signals}[1] }
-			: croak "Unknown \"signals\" value $cfg->{signals}"
+			$rt eq 'HASH'? $cfg->{$_}
+			: $rt eq 'ARRAY'? { verbose => $cfg->{$_}[0], quiet => $cfg->{$_}[1] }
+			: croak "Unknown \"$_\" value $cfg->{$_}"
 		} );
-	}
+	} for qw: signals handle_signals install_signal_handlers :;
 		
 	$self->format($cfg->{format})
 		if defined $cfg->{format};
 	
-	$self->output($cfg->{output})
-		if defined $cfg->{output};
+	defined $cfg->{$_} and $self->output($cfg->{$_})
+		for qw: out output :;
 	
 	$self->writer($cfg->{writer})
 		if defined $cfg->{writer};
@@ -424,7 +430,7 @@ sub _build_writer_code {
 	
 	if (ref $format eq 'CODE') {
 		# Closure over '$format', rather than deparsing the coderef
-		$code .= ' map {; $format->($_) } split /\n/, $message)';
+		$code .= ' map {; $format->($adapter, $level) } split /\n/, $message)';
 	} else {
 		$code .= ' map {; '.$format.' } split /\n/, $message)'; 
 	}
@@ -432,7 +438,7 @@ sub _build_writer_code {
 	return $code;
 }
 
-# "Cached Adapters" would more properly be a field of the config object, but then
+# "Cached Adapters" is conceptually a field of the config object, but then
 # it shows a giant mess if/when you Dump() the object, so I'm using this trick
 # to keep the list attached to the package instead of the object.
 # Object destructor cleans up the list.
@@ -478,28 +484,35 @@ Log::Any::Adapter::Daemontools::Config - Dynamic configuration settings used by
 
 =head1 VERSION
 
-version 0.090000_003
+version 0.090000_004
 
 =head1 SYNOPSIS
 
+  # As many config objects as you need
   my $cfg= Log::Any::Adapter::Daemontools->new_config;
-  my $cfg2= Log::Any::Adapter::Daemontools->new_config;
-  Log::Any::Adapter->set({ category => qr/^Foo::Bar/ }, 'Daemontools', config => $cfg );
-  Log::Any::Adapter->set({ category => qr/^Baz/ }, 'Daemontools', config => $cfg2 );
+  my $cfg2= Log::Any::Adapter::daemontools->new_config;
   
-  # Change log level independently
+  # Assign them to logging categories
+  Log::Any::Adapter->set({ category => qr/^Foo::Bar/ }, 'Daemontools', config => $cfg );
+  Log::Any::Adapter->set({ category => qr/^Baz/ },      'Daemontools', config => $cfg2 );
+  
+  # Update them later and existing adapters use the new settings
   $cfg->log_level('info');
   $cfg2->log_level('debug');
 
 =head1 DESCRIPTION
 
-When you use ..::Daemontools as the adapter for Log::Any, it creates one adapter
-per logging category.  In order to be able to change the logging level on the fly,
-we need shared variables that the adapters reference.
+When you use L<Log::Any::Adapter::Daemontools> as the adapter for L<Log::Any>,
+it creates one adapter per logging category.  If you set the log level as an
+attribute of the adapter, each adapter will have a copy and there is no
+practical way to reach out and change them. (other than resetting the adapters
+with a new configuration, which is a slightly expensive operation, and awkward)
+So, to make dynamic configuration changes easy, adapters reference these Config
+objects which hold the shared state.
 
-Objects of this class are "config instances".  There is often only one
-(Log::Any::Adapter::Daemontools->global_config) but you can create as many as you
-want in order to handle different categories separately.
+There is one default Config instance (L<global_config|Log::Any::Adapter::Daemontools/global_config>)
+but you can create as many as you want in order to handle different categories
+separately.
 
 =head1 ATTRIBUTES
 
@@ -510,12 +523,15 @@ want in order to handle different categories separately.
   $config->log_level( 99 );     # 'trace' (clamped to max)
   $config->log_level( 3 );      # 'error'
   $config->log_level( '+= 1' ); # 'warning'
-  $config->log_level( '-= 9' ); # 'emergency' (clamped to min)
+  $config->log_level( '-= 9' ); # 'none' (clamped to min)
 
 Get or set the current log level.  Can be assigned with either a name
-or a number, or "+= N" syntax.  (but use L<log_level_adjust> for that)
+or a number, or C<"+= N"> syntax.  (but use L</log_level_adjust> for that)
 Returns the level name.
 
+Accepts any of the names and aliases defined in L<Log::Adapter::Any::Util>
+but also the string C<'none'> (and value -1) to completely stop all logging.
+
 =head2 log_level_min
 
   # Our app should never have 'critical' squelched no matter how many '-q'
@@ -531,35 +547,36 @@ Returns the level name.
 
 Get or set the maximum allowed log level.
 Log levels above the maximum are silently clamped.
-Can be assigned either a name or number, or "+= N" syntax.
+Can be assigned either a name or number, or C<"+= N"> syntax.
 Returns the level name.
 
 =head2 output
 
-This is the handle (or coderef) log messages are written to.  If it is a
-handle (either a GLOB ref or class which can ->print) we call ->print on it.
-If it is a coderef, we pass the same arguments that would be given to print.
+This is the handle (or coderef) log messages are written to.  (but if you set
+L</writer> to a custom coderef, this attribute is ignored completely)
+
+If it is a handle (either a GLOB ref or class which C<can('print')>) we call its
+C<print> method.
+If it is a coderef, we pass the same arguments that would be given to C<print>.
 
 If a message from Log::Any contains newlines, they are broken into separate
-strings (but still ending with newline) so be ready to receive multiple
-arguments.
+strings (but still ending with newline) and passed as a list, so the print
+method or coderef should print its entire argument list.
 
-(But if you set L<writer> to a custom coderef, this attribute is ignored
- completely)
-
-The default is \*STDERR.
+The default is C<\*STDERR>.
 
 =head2 format
 
-This attrbute determines how messages are formatted into text lines.  It is a
-*string* of perl code (or a coderef) which executes within the following context:
+  $cfg->format( '$level eq "trace"? "$level: $_ at $file_brief line $line" : "$level_prefix$_"' )
+
+This attrbute determines how messages are formatted into text lines.
+(But if you set L</writer> to a custom coderef, this attribute is ignored completely.)
+
+It is a B<string of perl code> (or a coderef) which executes within the following context:
 
   $output->print( map { eval $format } split /\n/, $message );
 
-(But if you set L<writer> to a custom coderef, this attribute is ignored
- completely)
-
-The following variables become available if seen in your string of code:
+The following variables and functions are available for your string of code:
 
 =over
 
@@ -577,40 +594,50 @@ The name of the log level
 
 =item $level_prefix
 
-The default behavior of "$level: " for all levels except info which is ''.
+The default prefix behavior (C<"$level: "> for all levels except info, which is an empty string)
+
+=item numeric_level(...)
+
+The utility method from L<Log::Any::Adapter::Util>, useful for comparing levels.
 
 =item $file
 
-The full filename from caller()
+The full filename from C<caller()>
 
 =item $file_brief
 
-The filename from caller() minus the library path (best guess, no guarantees)
+The filename from C<caller()> minus the library path (best guess, no guarantees)
 
 =item $line
 
-The line number from caller()
+The line number from C<caller()>
 
 =item $package
 
-The package name from caller()
+The package name from C<caller()>
 
 =back
 
-The default is C<'"$level_prefix$_\n"'>
+If you use a coderef, you get C<$_> (a single line of the message), and arguments of
+C<($adapter, $level)>.  It should return a string ending with newline.
+
+  sub { my ($adapter, $level)= @_; return "$_\n"; }
+
+The default value is C<'"$level_prefix$_\n"'>
 
 =head2 writer
 
 If specified, this coderef overrides the routine that would have been built
-from L<output> and L<format>.
+from L</output> and L</format>.
 
 Its arguments are:
 
   sub { my ($adapter, $level, $message)= @_; ... };
 
 where $level is the level name, and adapter is an instance of
-Log::Any::Adapter::Daemontools.  (and the adapter attributes you are probably
-most interested in are 'category' and 'config').
+L<Log::Any::Adapter::Daemontools>.  (and the adapter attributes you are probably
+most interested in are L<category|Log::Any::Adapter::Daemontools/category> and
+L<config|Log::Any::Adapter::Daemontools/config>).
 
 =head1 METHODS
 
@@ -626,15 +653,20 @@ Different from a constructor, this method takes a hashref of short aliases
 and notations and calls various methods that might have effects outside of
 this object.
 The primary purpose is to provide convenient initialization of the global
-logging configuration.  The following are provided:
+logging configuration.
+
+DO NOT pass un-sanitized user input to init(), because the 'format' attribute
+is processed as perl code.
+
+The following are provided:
 
 =over
 
-=item env
+=item env, or process_env
 
   env => $name_or_args
 
-Convenient passthrough to L<process_env> method.
+Convenient passthrough to L</process_env> method.
 
 If env is a hashref, it is passed directly.  If it is a scalar, it is
 interpreted as a pre-defined "profile" of arguments.
@@ -649,11 +681,11 @@ Profiles:
 
 =back
 
-=item argv
+=item argv, or process_argv
 
   argv => $name_or_args
 
-Convenient passthrough to L<process_argv>.
+Convenient passthrough to L</process_argv>.
 
 If argv is a hashref, it is passed directly.  If it is a scalar, it is
 interpreted as a pre-defined "profile" of arguments.
@@ -675,54 +707,54 @@ Profiles:
 
 =back
 
-=item signals
+=item signals, handle_signals, or install_signal_handlers
 
   signals => [ $v, $q ],
   signals => { verbose => $v, quiet => $q },
 
-Convenient passthrough to L<install_signal_handlers>.
+Convenient passthrough to L</install_signal_handlers>.
 
-If handle_signals is an arrayref of length 2, they are used as the verbose and
+If signals is an arrayref of length 2, they are used as the verbose and
 quiet parameters, respectively.  If it is a hashref, it is passed directly.
 
-=item level
+=item level, or log_level
 
-Sets L<log_level>
+Sets L</log_level>
 
-=item min
+=item min, level_min, or log_level_min
 
-Sets L<log_level_min>
+Sets L</log_level_min>
 
-=item max
+=item max, level_max, or log_level_max
 
-Sets L<log_level_max>
+Sets L</log_level_max>
 
 =item format
 
-Sets L<format>
+Sets L</format>
 
-=item out
+=item out, or output
 
-Sets L<output>
+Sets L</output>
 
 =item writer
 
-Sets L<writer>
+Sets L</writer>
 
 =head2 log_level_num
 
-The current log level, returned as a number.  This is *NOT* a writeable attribute, just
-a shortcut for numeric_level( $self->log_level )
+The current log level, returned as a number.  This is B<NOT> a writeable attribute, just
+a shortcut for C<< numeric_level( $self->log_level ) >>.
 
 =head2 log_level_min_num
 
-The current minimum, returned as a number.  This is *NOT* a writeable attribute, just
-a shortcut for numeric_level( $self->log_level_min ).
+The current minimum, returned as a number.  This is B<NOT> a writeable attribute, just
+a shortcut for C<< numeric_level( $self->log_level_min ) >>.
 
 =head2 log_level_max_num
 
-The current maximum, returned as a number.  This is *NOT* a writeable attribute, just
-a shortcut for numeric_level( $self->log_level_max ).
+The current maximum, returned as a number.  This is B<NOT> a writeable attribute, just
+a shortcut for C<< numeric_level( $self->log_level_max ) >>.
 
 =head2 log_level_adjust
 
@@ -739,9 +771,9 @@ negative decreases it)
   $config->process_env( log_level => $ENV_VAR_NAME );
 
 Request that this package check for the named variable(s), and if set,
-interpret it either as a debug level or a log level, and then set L<log_level>.
+interpret it either as a debug level or a log level, and then set L</log_level>.
 
-A log_level environment variable is applied directly to the L<log_level> attribute.
+A log_level environment variable is applied directly to the L</log_level> attribute.
 
 A "debug level" environment variable refers to the typical Unix practice of a
 variable named DEBUG where 0 is disabled, 1 is enabled, and larger numbers
